import sys
from PySide6.QtWidgets import *
from types import SimpleNamespace

from puyadl.scraper import Scraper
from puyadl.dialog import showSimpleDialog, showErrorDialog


class Form(QWidget):
    
    def __init__(self, parent=None):
        super(Form, self).__init__(parent)
        self.setWindowTitle("puya-dl")
        self.resize(440, 330)

        self.mode = QGroupBox()
        mode1 = QRadioButton("PuyaSubs")
        mode1.setChecked(True)
        mode2 = QRadioButton("All")

        modeLayout = QHBoxLayout()
        modeLayout.addWidget(mode1)
        modeLayout.addWidget(mode2)

        self.mode.setLayout(modeLayout)
        mode1.toggled.connect(self.modeChange)

        self.warningLabel = QLabel("It's recommended to specify the group name and quality in the search query in this mode.")
        self.warningLabel.setStyleSheet("background-color: #263b45; color: white; border: 2px solid #3daee9; padding: 5px; border-radius: 3px;")
        self.warningLabel.setWordWrap(True)
        self.warningLabel.hide()

        # TopLayout (Title LineEdit + Quality ComboBox)
        self.title = QLineEdit()

        self.label = QLabel("Quality:")

        self.cb = QComboBox()
        self.cb.addItems(["1080p", "720p", "Unspecified"])

        self.topLayout = QHBoxLayout()
        self.topLayout.addWidget(self.title)
        self.topLayout.addWidget(self.label)
        self.topLayout.addWidget(self.cb)

        self.eps = QLineEdit()
        self.eps.setPlaceholderText("Episodes")
        self.eps.setDisabled(True)

        self.epsCheckBox = QCheckBox("Specify episodes to download")
        self.epsCheckBox.stateChanged.connect(self.checkboxEvent)

        self.confirmCheckbox = QCheckBox("Don't ask for confirmation")

        self.epsGroupLayout = QVBoxLayout()

        self.epsLayout = QHBoxLayout()
        self.epsLayout.addWidget(self.epsCheckBox)
        self.epsLayout.addWidget(self.eps)

        self.epsGroupLayout.addLayout(self.epsLayout)
        self.epsGroupLayout.addWidget(self.confirmCheckbox)

        self.epsGroup = QGroupBox()
        self.epsGroup.setLayout(self.epsGroupLayout)

        self.button = QPushButton("Download")

        self.progress = QProgressBar()
        self.progress.setTextVisible(True)

        layout = QVBoxLayout(self)
        layout.addWidget(self.mode)
        layout.addWidget(self.warningLabel)
        layout.addLayout(self.topLayout)
        layout.addWidget(self.epsGroup)

        layout.addStretch(1)
        layout.addWidget(self.button)
        layout.addWidget(self.progress)

        self.button.clicked.connect(self.query)

    def modeChange(self, state):
        self.puya = state
        self.label.setVisible(state)
        self.cb.setVisible(state)
        self.warningLabel.setVisible(not state)

    def checkboxEvent(self, state):
        if state == 0:
            self.eps.setDisabled(True)
        else:
            self.eps.setDisabled(False)

    def choiceDialog(self, titles):
        dialog = QDialog()
        dialog.resize(260, 260)
        vbox = QVBoxLayout(dialog)

        label = QLabel("Multiple titles have been found. Please select which one to download.")
        btnGroup = QButtonGroup(vbox)

        vbox.addWidget(label)

        buttons = []

        for i, title in enumerate(titles):
            button = QRadioButton(title)
            btnGroup.addButton(button)
            btnGroup.setId(button, i)

            vbox.addWidget(button)
            buttons.append(button)

        buttons[0].setChecked(True)

        hbox = QHBoxLayout()
        cancel = QPushButton("Cancel")
        ok = QPushButton("Confirm")

        hbox.addWidget(cancel)
        hbox.addWidget(ok)

        vbox.addStretch(1)
        vbox.addLayout(hbox)
        
        cancel.clicked.connect(lambda: dialog.reject())
        ok.clicked.connect(lambda: self.dialogClose(dialog, btnGroup))
        
        dialog.setWindowTitle("puya-dl")
        dialog.setModal(True)

        return dialog

    def dialogClose(self, dialog, group):
        dialog.done(group.checkedId()+1) # +1 because 0 means no choice at all

    def query(self):
        if len(self.title.text()) == 0:
            showErrorDialog("No query specified")
            return

        self.progress.setValue(0)
        
        args = SimpleNamespace()
        quality = self.cb.currentText()
        args.quality = quality if quality != "Unspecified" else ""
        args.episodes = self.eps.text() if self.epsCheckBox.isChecked() else None
        args.all = not self.puya

        self.progress.setFormat("Fetching results from nyaa...")
        self.progressTo(0, 25)
        scraper = Scraper(args)
        scraper.request(self.title.text()) # TODO exception handling

        if len(scraper.items) == 0:
            showErrorDialog("No results found")
            self.cancel()
            return

        self.progress.setFormat("Parsing results...")
        self.progressTo(25, 50)

        titles = scraper.list_titles()
        if len(titles) > 1:
            choice = self.choiceDialog(titles)
            result = choice.exec_()
            if result == 0:
                print("No choice")
                self.progress.setValue(0)
                self.cancel()
                return
            else:
                index = result - 1
        else:
            index = 0

        self.progress.setFormat("Opening magnet links...")
        scraper.filter(titles[index])
        scraper.downloadFirstItem()
        if self.confirmCheckbox.isChecked() or showSimpleDialog("Your BitTorrent client should open with the first file. Hit OK to continue.") == QMessageBox.Ok:
            self.progressTo(50, 100)
            scraper.download()
            self.progress.setFormat("Your BitTorrent client should open.")
        else:
            self.cancel()

    def cancel(self):
        self.progress.setFormat("Ready")
        self.progress.setValue(0)

    def progressTo(self, start, to):
        completed = start
        while completed < to:
            completed += 0.0001
            self.progress.setValue(completed)

def initialize():
    app = QApplication(sys.argv)
    form = Form()
    form.show()
    sys.exit(app.exec_())


from puyadl.episode_parser import parseEpisodeFilter
import argparse
import sys
import os
import subprocess
import re
import requests
from bs4 import BeautifulSoup

def multiplatformOpen(destination):
    if sys.platform == "win32" or sys.platform == "cygwin":
        os.startfile(destination)
    elif sys.platform == "darwin":
        subprocess.run(['open', destination], stderr=subprocess.DEVNULL)
    else:
        subprocess.run(['xdg-open', destination], stderr=subprocess.DEVNULL)

class Scraper:

    def __init__(self, args):
        self.args = args

    def request(self, query):
        if self.args.all:
            res = requests.get("https://nyaa.si/?q="+query, headers={'User-Agent': 'puya-dl/1.0'})
        else:
            res = requests.get("https://nyaa.si/user/puyero?q="+query+"+"+self.args.quality, headers={'User-Agent': 'puya-dl/1.0'})
        parsed = BeautifulSoup(res.content, 'html.parser')

        self.items = []

        results = parsed.find_all('tr') # Find every row
        if len(results) == 0:
            print("No results found.")
            return
                    
        for result in results[1:]:
            links = result.select('td a')
            if "comments" in links[1]['href']:
                title = links[2]['title']
            else:
                title = links[1]['title'] # Title
            
            print(title)
            p = re.compile(r'(?P<group>\[.*\])\s(?P<title>.*)\s-\s(?P<episode>\d+)')
            m = p.search(title)
            if not m:
                print("No match")
                continue
            ep = {
                "title": m.group('title'),
                "episode": m.group('episode'),
                "magnet": links[-1]['href']
            }
            self.items.append(ep)

        # return items

    def list_titles(self):
        unq_list = []
        for x in self.items:
            if x['title'] not in unq_list:
                unq_list.append(x['title'])

        return unq_list

    def filter(self, title):
        filtered = []
            
        if self.args.episodes:
            episodes = parseEpisodeFilter(self.args.episodes)
            for x in self.items:
                if x['title'] == title:
                    try:
                        ep = int(x['episode'])
                        if ep in episodes:
                            filtered.append(x)
                    except:
                        print("Couldn't parse episode number: ", x['episode'])
        else:
            for x in self.items:
                if x['title'] == title:
                    filtered.append(x)

        self.items = filtered
    
    def downloadFirstItem(self):
        multiplatformOpen(self.items[-1]['magnet'])

    def download(self):
        for x in self.items[::-1][1:]:
            multiplatformOpen(x['magnet'])


from PySide6.QtWidgets import QMessageBox

def showSimpleDialog(message):
    box = QMessageBox()
    box.setIcon(QMessageBox.Question)

    box.setText(message)
    box.setWindowTitle("puya-dl")
    box.setStandardButtons(QMessageBox.Cancel | QMessageBox.Ok)
    
    return box.exec_()

def showErrorDialog(message):
    box = QMessageBox()
    box.setIcon(QMessageBox.Critical)
    box.setText(message)
    box.setWindowTitle("puya-dl")
    box.setStandardButtons(QMessageBox.Ok)
    
    return box.exec_()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="puya-dl",
    version="1.1.2",
    author="pizza61",
    author_email="2006mappa@protonmail.com",
    description="Short Python script and GUI for batch downloading PuyaSubs releases",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pizza61/puya-dl",
    project_urls={
        "Bug Tracker": "https://github.com/pizza61/puya-dl/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Environment :: X11 Applications :: Qt"
    ],
    packages=['puyadl'],
    install_requires=[
        'requests>=2.25.1',
        'beautifulsoup4>=4.9.3',
        'PySide6>=6.0.2'
    ],
    python_requires=">=3.6"
)


from puyadl.scraper import Scraper
from puyadl.gui import initialize
import sys
import argparse

def download(scraper, noconfirm):
    if noconfirm:
        scraper.downloadFirstItem()
        scraper.download()
    else:
        print("\033[93mYour BitTorrent client should open with the first file. Please specify a directory so your client remembers it. Type 'cancel' if you want to abort or anything else to continue.\033[0m")
        scraper.downloadFirstItem()
        a = input("(cancel/continue)> ").lower()
        if a == 'cancel':
            print("Cancel")
        else:
            print("\033[93mContinuing.\033[0m")
            scraper.download()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog="puyadl", description="puya.moe batch download tool")
    parser.add_argument('title', nargs='+', help="Exact title")
    parser.add_argument('-q', '--quality', dest="quality", help="Quality (usually only 720p and 1080p is available)", default="1080p")
    parser.add_argument('-e', '--episodes', dest="episodes", help="Specify episodes to download", required=False)
    parser.add_argument('--dryrun', action='store_true', dest="dryrun", help="Dry run (only for development)")
    parser.add_argument('--quiet', '--noconfirm', action='store_true', dest="noconfirm", help="Don't ask for confirmation")
    parser.add_argument('--all', action='store_true', dest="all", help="Search for all releases (not only puya) (experimental)")

    if len(sys.argv) > 1: # Arguments provided, CLI mode
        args = parser.parse_args()
        query = ' '.join(args.title)

        if args.dryrun:
            exit()
        
        scraper = Scraper(args)
        scraper.request(query)
        titles = scraper.list_titles()

        if len(titles) > 1:
            print("Multiple titles found. Please select which one you want to download")
            for i, title in enumerate(titles):
                print(i, ")", title)
            output = int(input("Number > "))
            if output > len(titles) or output < 0:
                print("Incorrect number. Exiting")
                exit(0)
            else:
                print("Selected", titles[output])

                scraper.filter(titles[output])
                download(scraper, args.noconfirm)    
        else:
            scraper.filter(titles[0])
            download(scraper, args.noconfirm)

        print("\033[93mFinished.\033[0m")

    else: # Arguments not provided, initialize gui
        print("Initializing Qt6 gui")
        initialize()


def parseEpisodeFilter(episodes):
    filterList = episodes.split(',')
    allowedEps = []
    # Two types of filters are allowed:
    # range (x-y) i.e. 1-3
    # single number i.e. 2
    for f in filterList:
        try:
            fInt = int(f)
            allowedEps.append(fInt)
        except:
            if "-" in f:
                ranges = f.split('-')
                if len(ranges) == 2:
                    x = int(ranges[0])
                    y = int(ranges[1])
                    if x == y:
                        allowedEps.append(x)
                    elif y > x:
                        for i in range(x, y+1):
                            allowedEps.append(i)
                    else:
                        print("This program is too dumb to interpret this")
                else:
                    print(f, "is invalid episode filter")
            else:
                print(f, "is invalid episode filter")
    return allowedEps